% A backtracking routine to show the solutions to the pentomino puzzle 
clear; close all; clc

% Initialize colormap
map = [0 0 1;         % blue
       0 1 0;         % green
       1 0 0;         % red
       0.5 0.5 0.5;   % gray 
       0.5 0.25 0;    % brown
       1 0 1;         % magenta
       0 1 1;         % cyan
       0.5 0 0.5;     % purple
       0  0.5 0;      % dark green
       1 1 0;         % yellow
       1 0.5 0;       % orange
       0.25 0.75 1;   % lightblue
       0 0 0];        % black

% Set patternlist
P = [
1,2,3,4,7;1,2,3,16,11;1,2,3,17,10;1,2,3,19,11;1,2,3,18,10;1,2,15,16,9; 
1,2,16,17,12;1,2,16,18,2;1,2,16,32,5;1,2,17,18,12;1,2,17,33,3;1,2,18,19,9;
1,2,18,34,5;1,14,15,16,9;1,15,16,17,12;1,15,16,31,6;1,15,16,32,8; 
1,16,17,18,12;1,16,17,32,12;1,16,17,33,12;1,16,32,31,4;1,16,32,33,2;
1,16,32,48,11;1,17,18,19,9;1,17,18,33,8;1,17,18,34,6;1,17,32,33,2;
1,17,33,34,4;1,17,33,49,11;16,17,18,19,11;16,17,18,32,3;16,17,18,33,8;
16,17,18,34,4;16,17,31,32,8;16,17,32,33,12;16,17,32,48,10;16,17,33,34,6;
16,17,33,49,9;16,30,31,32,5;16,31,32,33,3;16,31,32,47,9;16,31,32,48,10; 
16,32,33,34,5;16,32,33,48,10;16,32,33,49,9;16,32,47,48,11;16,32,48,49,11;
16,32,48,64,7;15,16,17,18,10;15,16,17,31,8;15,16,17,32,1;15,16,17,33,8;
15,16,30,31,6;15,16,31,32,12;15,16,31,47,9;15,16,32,33,8;15,16,32,48,10; 
14,15,16,17,10;14,15,16,30,4;14,15,16,31,8;14,15,16,32,3;2,16,17,18,2;
13,14,15,16,11];   
   
% Initialize superfield
super_field = 13 * ones(16, 32);
% Initialize subfield (puzzle) which has to lie in the superfield 
sub_field = [10, 6]; % r = [20, 3]; r = [15, 4]; r = [8, 8];

super_field(1:sub_field(2), 1:sub_field(1)) = 0;
if sub_field(1) == 8 && sub_field(2) == 8 
   for s = 0 : 16 : 16
      super_field(52 + s : 53 + s) = 13;
   end
end

% Plot the subfield
set(gcf, 'Color', 'w')
plothandle = pcolor(super_field(1 : 1 + sub_field(2), 1 : 1 + sub_field(1)));
set(plothandle, 'EdgeColor', 'none');
axis('square', 'equal', 'off');
colormap(map)

% Solve
animation = false; % If set to true, it wil show the process.
tic
counter = 0;
% I stores the patterns
I = []; 
% label stores the label-coordinates of the patterns from I
label = find(super_field == 0, 1); 
COORDINATES = [];
i = 1; 
while i <= 64      
      L = sum(super_field(P(i : end, 1 : 4) + label(end)) == 0, 2);
      L = find(L == 4); % Candidates
      L = L + i - 1;
      L = L(ismember(P(L, 5), P(I, 5)) == 0);      
      if isempty(L)
         i = 64;
         % Termination
         if isempty(COORDINATES)
            break 
         else
            coordinates = COORDINATES(end, :);
            super_field(coordinates) = 0;
            % Update COORDINATES
            COORDINATES = COORDINATES(1:end-1, :);
            % Reset Label
            label = find(super_field == 0, 1); 
            % Backtrack
            i = I(end) + 1;
            I = I(1:end - 1); 
         end
      else
         i = L(1);
         coordinates = [label(end), label(end) + P(i, 1:4)];
         super_field(coordinates) = P(i, 5);
         COORDINATES = [COORDINATES; coordinates];
         label = [label, find(super_field == 0, 1)];
         I = [I, i]; 
         i = 1; 
      end
   % Print process and solutions
   if animation
      set(plothandle, 'CData', super_field(1 : 1 + sub_field(2), 1 : 1 + sub_field(1)));
      colormap(map)
      drawnow
   end
   if ~numel(find(super_field == 0))
      set(plothandle, 'CData', super_field(1 : 1 + sub_field(2), 1 : 1 + sub_field(1)));      
      counter = counter + 1;
      title(int2str(counter)) 
      drawnow   
      % Remove last pattern in order to find the next solution
      i = 64;     
   end
end
toc